@@ -756,22 +756,34 @@ sub on_wrap_method($)
     }
   }
 
-  # Extra ref needed?
-  while(scalar(@args) > 2) # If the optional ref/err arguments are there.
+  # Extra stuff needed?
+  $$objCfunc{deprecated} = "";
+  my $deprecation_docs = "";
+  while(scalar(@args) > 2) # If the optional ref/err/deprecated arguments are there.
   {
     my $argRef = string_trim(pop @args);
+    #print "debug arg=$argRef\n";
     if($argRef eq "refreturn")
     {
       $$objCfunc{rettype_needs_ref} = 1;
     }
-      if($argRef eq "errthrow")
+    elsif($argRef eq "errthrow")
     {
       $$objCfunc{throw_any_errors} = 1;
     }
+    elsif($argRef =~ /^deprecated(.*)/) #If deprecated is at the start.
+    {
+      $$objCfunc{deprecated} = "deprecated";
+
+      if($1 ne "")
+      {
+        $deprecation_docs = string_unquote(string_trim($1));
+      }
+    }
   }
-   
+
   my $commentblock = "";
-  $commentblock = DocsParser::lookup_documentation($argCFunctionName);
+  $commentblock = DocsParser::lookup_documentation($argCFunctionName, $deprecation_docs);
 
   $objOutputter->output_wrap_meth($filename, $line_num, $objCppfunc, $objCfunc, $argCppMethodDecl, $commentblock);
 }
@@ -808,7 +820,7 @@ sub on_wrap_method_docs_only($)
   $argCFunctionName = string_trim($argCFunctionName);
 
   #Get the c function's details:
-  
+
   #Checks that it's not empty and that it contains no whitespace.
   if ($argCFunctionName =~ /^\S+$/ ) 
   {
@@ -833,7 +845,7 @@ sub on_wrap_method_docs_only($)
   }
 
   my $commentblock = "";
-  $commentblock = DocsParser::lookup_documentation($argCFunctionName);
+  $commentblock = DocsParser::lookup_documentation($argCFunctionName, "");
 
   $objOutputter->output_wrap_meth_docs_only($filename, $line_num, $commentblock);
 }
@@ -934,9 +946,11 @@ sub on_wrap_signal($)
   my $bCustomDefaultHandler = 0;
   my $bNoDefaultHandler = 0;
   my $bCustomCCallback = 0;
-  if(scalar(@args) > 2) # If optional arguments are there.
+  my $bRefreturn = 0;
+  
+  while(scalar(@args) > 2) # If optional arguments are there.
   {
-    my $argRef = string_trim($args[2]);
+    my $argRef = string_trim(pop @args);
     if($argRef eq "custom_default_handler")
     {
       $bCustomDefaultHandler = 1;
@@ -951,10 +965,15 @@ sub on_wrap_signal($)
     {
       $bCustomCCallback = 1;
     }
+
+    if($argRef eq "refreturn")
+    {
+      $bRefreturn = 1;
+    }
   }
 
 
-  $self->output_wrap_signal( $argCppDecl, $argCName, $$self{filename}, $$self{line_num}, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback);
+  $self->output_wrap_signal( $argCppDecl, $argCName, $$self{filename}, $$self{line_num}, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback, $bRefreturn);
 }
 
 # void on_wrap_vfunc()
@@ -1077,6 +1096,12 @@ sub on_wrap_property($)
   my $argPropertyName = $args[0];
   $argPropertyName = string_trim($argPropertyName);
   $argPropertyName = string_unquote($argPropertyName);
+
+  #Convert the property name to a canonical form, as it is inside gobject.
+  #Otherwise, gobject might not recognise the name, 
+  #and we will not recognise the property name when we get notification that the value changes.
+  $argPropertyName =~ s/_/-/g; #g means replace all.
+
   my $argCppType = $args[1];
   $argCppType = string_trim($argCppType);
   $argCppType = string_unquote($argCppType);
@@ -1110,12 +1135,12 @@ sub output_wrap_check($$$$$$)
 
 }
 
-# void output_wrap($CppDecl, $signal_name, $filename, $line_num, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback)
+# void output_wrap($CppDecl, $signal_name, $filename, $line_num, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback, $bRefreturn)
 # Also used for vfunc.
-sub output_wrap_signal($$$$$$$)
+sub output_wrap_signal($$$$$$$$)
 {
-  my ($self, $CppDecl, $signal_name, $filename, $line_num, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback) = @_;
-
+  my ($self, $CppDecl, $signal_name, $filename, $line_num, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback, $bRefreturn) = @_;
+  
   #Some checks:
   $self->output_wrap_check($CppDecl, $signal_name, $filename, $line_num, "WRAP_SIGNAL");
 
@@ -1154,7 +1179,7 @@ sub output_wrap_signal($$$$$$$)
 
     my $bImplement = 1;
     if($bCustomDefaultHandler) { $bImplement = 0; }
-    $objOutputter->output_wrap_default_signal_handler_cc($filename, $line_num, $objCppSignal, $objCSignal, $bImplement, $bCustomCCallback);
+    $objOutputter->output_wrap_default_signal_handler_cc($filename, $line_num, $objCppSignal, $objCSignal, $bImplement, $bCustomCCallback, $bRefreturn);
   }
 }
 
